#!/usr/bin/env python

# The PyTone configuration file is split into three parts I -- III.
#
# In part I you most likely want to change the basedir option.
#
# In part IIa you can change the configuration of
# the player(s) you want to use.
# In part IIb you can change the list of active players.
#
# Part III, where the keybindings are defined, you probably want
# to leave alone for the moment.

##############################################################################
# I. general PyTone configuration
##############################################################################

# basedir: basedir for filesystem based navigation and auto
# registering of songs in database.
#
# !!! You almost certainly want to adjust this !!!
basedir = "/mnt/mp3"

# songdbbase: base name of songdb
#
# The song database stores all relevant informations of your songs.
# Here you can specify the prefix of the files it uses. The actual files
# are named "<songdbbase>_songs.db" and "<songdbbase>_artists.db"
songdbbase = "~/mp3"

# logfile: Name of the file, where the played songs are logged.
#
# Set this to None, if you don't want the logging.
logfile = "~/pytone.log"

# here you can configure, whether the filelist and the playlist window, 
# respectively, have scrollbars
filelistwinhasscrollbar = 1
playlistwinhasscrollbar = 0

##############################################################################
# IIa. player configuration
#
# general format: 
#   name:    name of player
#   options: dictionary with player options
##############################################################################

##############################################################################
# mad: internal player (using libmad)
#
# options:
#   device:              libao device ("oss", "alsa", "alsa09", "esd", etc.)
#   dsp:                 path of dsp used for output
#   bufsize:             0:  no buffering by PyTone
#                        >0: size of internal buffer used for output in kBytes
#                        Note: A large buffer may prevent clicking but
#                        leads to a delayed response for instance to play next
#                        song requests
#   autoplay:            automatic start of playing, if songs are in playlist 
#   crossfadingstart:    start crossfading, if currently played song has only
#                        mixingstart seconds left
#   crossfadingduration: duration of crossfading in seconds
##############################################################################

# define some example madplayers
# the first one uses /dev/dsp (with a 500 kByte large buffer) 
madplayer_oss_0 = {
    "name"   : "mad",
    "options": {"aodevice"           : "oss",
                "aooptions"          : {"dsp": "/dev/dsp"},
                "bufsize"            : 500,
                "autoplay"           : 1, 
                "crossfadingstart"   : 5,
                "crossfadingduration": 6}
    }


# the second one outputs to  /dev/dsp1 and is unbuffered
madplayer_oss_1 = {
    "name"   : "mad",
    "options": {"aodevice"           : "oss",
                "aooptions"          : {"dsp": "/dev/dsp1"},
                "bufsize"            : 0,
                "autoplay"           : 1, 
                "crossfadingstart"   : 5,
                "crossfadingduration": 6}
    }

# here comes a madplayer which outputs with a small buffer to aRts
madplayer_arts = {
    "name"   : "mad",
    "options": {"aodevice"           : "arts",
                "aooptions"          : {},
                "bufsize"            : 100,
                "autoplay"           : 1, 
                "crossfadingstart"   : 5,
                "crossfadingduration": 6}
    }

##############################################################################
# mpg123: external mpg321 or mpg123 based player
#
# options:
#   autoplay: automatic start of playing, if songs are in playlist
#   cmdline:  cmdline use for the start of the player
##############################################################################

# define two mpg123 type external players (here mpg321) using /dev/dsp
# and /dev/dsp1
mpg123player0 = {
    "name"   : "mpg123",
    "options": {"cmdline" : "/usr/bin/mpg321 --skip-printing-frames=5 -a /dev/dsp",
                "autoplay": 1}
    }

mpg123player1 = {
    "name"   : "mpg123",
    "options": {"cmdline": "/usr/bin/mpg321 --skip-printing-frames=5 -a /dev/dsp1",
                "autoplay": 1}
    }

##############################################################################
# xmms: external player using xmms
#
# options:
#   session:  xmms session number
#   autoplay: automatic start of playing, if songs are in playlist
#   noqueue:  0=no internal queue for song
#             1=internal queue for songs, such that crossfading (via xmms-crossfade)
#               is possible
##############################################################################

# xmms player without automatic start of playing and without queueing
# for crossfading
xmmsplayer = {
    "name"   : "xmms",
    "options": {"session" : 0,
                "autoplay": 1, 
                "noqueue" : 0}
    }

##############################################################################
# IIb. players: defines the list of active players
# 
# Note:
#  If you specify only one player, it will be used for playing of the songs in
#  the playlist.
#  If you specify two players, the first one is used for the playlist and
#  the second one always plays the currently selected song. For this latter
#  option you obviously need two outputs on your soundcard or two cards.
##############################################################################

# comment in one of the following players definitions

# configurations with one player
players = [madplayer_arts]
# players = [mpg123player0]
# players = [xmmsplayer]

# a configuration with two players: the first one plays the songs in the
# playlist, the second one always plays the currently selected song
# use this, if you have to soundcards or one with two line outs

# players = [madplayer_oss_0, madplayer_oss_1]

##############################################################################
# III. key bindings
#
# dictionary with entries
#  "global":   global key bindings
#  "filelist": key bindings only active in filelist window
#  "playlist": key bindings only active in playlist window
# which map from
#   name of function to 
#   tuple of accepted keys
##############################################################################

# we need the curses module for the key codes
import curses

# define convenience function: ctrl keys
def ctrl(char):
    """return ascii value for ctrl+<char>"""
    return ord(char.upper())-64

keybindings = {
    "global": {
          "refresh":           (ctrl("l"),),
          "exit":              (ctrl("x"),),     # needs to pressed two times in fast succ.
          "playerstart":       (ord("p"), ord("P")),
          "playernextsong":    (ord("n"), ord("N")),
          "playerstop":        (ord("S"),),
          "playlistdeleteplayedsongs":
                               (curses.KEY_BACKSPACE,),
          "playlistclear":     (ctrl("d"),),
          "playlistsave":      (ctrl("w"),),
          "playlistload":      (ctrl("r"),),
    },
    "filelist": {
          "selectnext":        (curses.KEY_DOWN,),
          "selectprev":        (curses.KEY_UP,),
          "selectnextpage":    (ctrl("n"), curses.KEY_NPAGE),
          "selectprevpage":    (ctrl("p"), curses.KEY_PPAGE),
          "selectfirst":       (ctrl("a"), curses.KEY_HOME),
          "selectlast":        (ctrl("e"), curses.KEY_END),
          "dirdown":           (ord(" "), ord("\n"), curses.KEY_ENTER, curses.KEY_RIGHT),
          "dirup":             (curses.KEY_LEFT,),
          "addsongtoplaylist": (ord(" "), ord("\n"), curses.KEY_ENTER, curses.KEY_RIGHT),
          "adddirtoplaylist":  (ord("i"), ord("I"), curses.KEY_IC),
          "activateplaylist":  (ord("\t"),),
          "toggleview":        (ord("t"), ord("T")),
          "generaterandomlist":(ord("r"), ord("R")),
          "search":            (ord("/"), ctrl("s"))
          },
    "playlist": {
          "selectnext":        (curses.KEY_DOWN,),
          "selectprev":        (curses.KEY_UP,),
          "selectnextpage":    (ctrl("n"), curses.KEY_NPAGE),
          "selectprevpage":    (ctrl("p"), curses.KEY_PPAGE),
          "selectfirst":       (ctrl("a"), curses.KEY_HOME),
          "selectlast":        (ctrl("e"), curses.KEY_END),
          "moveitemup":        (ord("+"),),  
          "moveitemdown":      (ord("-"),),
          "deleteitem":        (ord("d"), ord("D"), curses.KEY_DC),
          "activatefilelist":  (ord("\t"), curses.KEY_LEFT, curses.KEY_RIGHT),
          }

    }
