#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import fcntl
import string

import events

# player states

STOP     = 0
WANTPLAY = 1
PAUSE    = 3 # not used yet
PLAY     = 4


def makeNonBlocking(fd):
    fl = fcntl.fcntl(fd, fcntl.F_GETFL)
    fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)

class player:
    def __init__(self,
                 cmdline="/usr/bin/mpg321 --skip-printing-frames=5 -a /dev/dsp",
                 autoplay=0):
        self.cmdline = cmdline
        self.autoplay = autoplay
        self.initmpg123()
        
        events.ec.subscribe(events.timer, self.timer)
        events.ec.subscribe(events.playerstart, self.playerstart)
        events.ec.subscribe(events.playerstop, self.playerstop)
        events.ec.subscribe(events.playerforward, self.playerforward)
        events.ec.subscribe(events.nextsong, self.nextsong)

    def initmpg123(self):
        self.pstdin, self.pstdout = os.popen4(self.cmdline + " -R -")
        assert self.pstdout.readline()=="@R MPG123\n", "Cannot initialize player"
        makeNonBlocking(self.pstdout.fileno())
        self.title = ""
        if self.autoplay:
            self.state = WANTPLAY
        else:
            self.state = STOP
        events.ec.notify(events.playbackinfochanged(self))

    def closempg123(self):
        if self.pstdin:
            self.pstdin.close()
        if self.pstdout:
            self.pstdout.close()

    def sendmpg123(self, command):
        try:
            self.pstdin.write("%s\n" % command)
            self.pstdin.flush()
        except IOError, error:
            # broken pipe => restart player
            if error[0]==32:
                self.closempg123()
                self.initmpg123()
                self.pstdin.write("%s\n" % command)
                self.pstdin.flush()
            else:
                raise

    def receivempg123(self):
        try:
            return self.pstdout.readline()
        except IOError:
            return ""
        
    # event handler methods

    def timer(self, event):
        """player update, regularly called by PyTone"""
        while 1:
            r = self.receivempg123()
            if r=="": break
            elif r.startswith("@F") and self.title!="":
                pframes, lframes, ptime, ltime = string.split(r[3:])
                ptime = float(ptime)*1000
                ltime = float(ltime)*1000
                events.ec.notify(
                    events.playbackinfochanged(self,
                                               1, self.title, ptime, ptime+ltime))
            elif r.startswith("@I ID3:"):
                self.title = r[37:67].strip() + " - " + r[7:37].strip()
                self.state = PLAY
            elif r.startswith("@P"):
                if self.state==PLAY and r[3]=="0":
                    events.ec.notify(events.requestnextsong(self))
                    
        if self.state==WANTPLAY:
            events.ec.notify(events.requestnextsong(self))
    
    def nextsong(self, event):
        """play event.song next"""
        if event.player is self:
            if event.song:
                self.sendmpg123("L %s" % event.song.path)
                self.title = event.song.name
            else:
                if self.autoplay:
                    self.state = WANTPLAY
                else:
                    self.playing = STOP
                events.ec.notify(events.playbackinfochanged(self))

    def playerstart(self, event):
        """start playing"""
        if event.player is self and self.state!=PLAY:
            events.ec.notify(events.requestnextsong(self))

    def playerstop(self, event):
        """stop playing"""
        if event.player is self:
            self.sendmpg123("S")
            self.title = ""
            self.state = STOP
            events.ec.notify(events.playbackinfochanged(self))

    def playerforward(self, event):
        """immediately play next song"""
        if event.player is self:
            events.ec.notify(events.requestnextsong(self))

    def playerquit(self):
        self.sendmpg123("Q")
        self.closempg123()
                    
            

