#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses
import os.path

import window
import events
import filelistwin
import playlistwin
import playerwin
import iteminfowin
import statusbar
import songdb
import players

import config

from helper import debug

class mainwin(window.window):
    def __init__(self, screen, h, w, y, x):
        window.window.__init__(self, screen, h, w, y, x, None)
        self.screen = screen
        leftpanelw = int(w/2.2)
        rightpanelw = w-leftpanelw

        config.basedir = os.path.expanduser(config.basedir)
        config.songdbbase = os.path.expanduser(config.songdbbase)
        config.logfile = os.path.expanduser(config.logfile)

        self.songdb = songdb.songdb(config.songdbbase)
        songdb.songautoregisterer(self.songdb, config.basedir)

        self.keybindings = config.keybindings["global"]

        self.statusbar = statusbar.statusbar(screen, h-1, w)
        
        self.filelistwin = filelistwin.filelistwin(screen,
                                                   h-1, leftpanelw, 0, 0,
                                                   config.filelistwinhasscrollbar,
                                                   config.basedir,
                                                   self.songdb,
                                                   config.keybindings["filelist"],
                                                   )

        self.playlistwin = playlistwin.playlistwin(screen,
                                                   h-6-3-1, rightpanelw, 9, leftpanelw,
                                                   config.playlistwinhasscrollbar,
                                                   self.songdb,
                                                   config.logfile,
                                                   config.keybindings["playlist"])

        self.players = []
        for player in config.players:
            name = player["name"]
            options = player["options"]
            if name=="xmms":
                import players.xmmsplayer
                self.players.append(players.xmmsplayer.player(**options))
            elif name=="mpg123":
                import players.mpg123player
                self.players.append(players.mpg123player.player(**options))
            elif name=="hip":
                import players.hipplayer
                self.players.append(players.hipplayer.player(**options))
            elif name=="mad":
                import players.madplayer
                self.players.append(players.madplayer.player(**options))

        self.playerwin = playerwin.playerwin(screen,
                                             3, rightpanelw, 0, leftpanelw,
                                             self.players[0],
                                             config.keybindings)

        try:
            sp = self.players[1]
        except IndexError:
            sp = None
        self.iteminfowin = iteminfowin.iteminfowin (screen,
                                                    6, rightpanelw, 3, leftpanelw,
                                                    sp)



        events.ec.subscribe(events.activateplaylist, self.activateplaylist)
        events.ec.subscribe(events.activatefilelist, self.activatefilelist)
        events.ec.subscribe(events.keypressed, self.keypressed)
        
        events.ec.notify(events.activewinchanged(self.filelistwin))


        curses.curs_set(0)
        curses.meta(1)
        curses.raw()
        curses.halfdelay(1)

        while 1:
            try:
                events.ec.notify(events.timer())

                key = self.screen.getch()
                
#                curses.flushinp()

                if key==27:
                    # handle escape sequence (e.g. alt+key)
                    key = self.screen.getch()+1024
                    
                if key in self.keybindings["exit"]:
                    curses.halfdelay(5)
                    key = self.screen.getch()
                    curses.halfdelay(1)
                    if key in self.keybindings["exit"]:
                        break
                elif key!=-1:
                    events.ec.notify(events.keypressed(key))
                    
                curses.doupdate()                                    

            except KeyboardInterrupt:
                pass
            
        for player in self.players:
            player.playerquit()

        self.songdb.close()

    # event handler

    def activateplaylist(self, event):
        events.ec.notify(events.activewinchanged(self.playlistwin))

    def activatefilelist(self, event):
        events.ec.notify(events.activewinchanged(self.filelistwin))

    def keypressed(self, event):
        key = event.key
        if key in self.keybindings["refresh"]:
            # refresh screen (i don't know any better way...)
            self.screen.clear()
            self.filelistwin.update()
            self.playlistwin.update()
            self.playerwin.update()
            self.iteminfowin.update()
            self.statusbar.update()
        elif key in self.keybindings["playlistdeleteplayedsongs"]:
            events.ec.notify(events.playlistdeleteplayedsongs())
        elif key in self.keybindings["playlistclear"]:
            events.ec.notify(events.playlistclear())
        elif key in self.keybindings["playlistload"]:
            events.ec.notify(events.playlistload())
        elif key in self.keybindings["playlistsave"]:
            events.ec.notify(events.playlistsave())
        else:
            debug("unknown key: %d\n" % key)

def main(stdscr):
    mw = mainwin(stdscr, curses.LINES, curses.COLS, 0, 0)
        
curses.wrapper(main)
