#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import events

class slist:
    """ Generic list class with selectable items

    List with selectable items, out of which maximally height are
    being displayed at the same time.
    
    If list is active, selectionchanged events are issued upon
    change of the currently selected item.
    """
    
    def __init__(self, height):
        self.height = height   # maximal number of displayed items
        self.items = []
        self.selected = None   # currently selected item
        self.top = 0           # first displayed item
        self.active = 0        # is list currently active?

    # generic list methods

    def __getitem__(self, key):
        return self.items[key]

    def __len__(self):
        return len(self.items)

    def __delitem__(self, index):
        del self.items[index]
        if self.selected is not None:
            if len(self)==0:
                self.selected = None
            else:
                if index<self.selected:
                    self.selected -= 1
                self.selected = min(self.selected, len(self)-1)
            self._notifyselectionchanged()
        self._updatetop()

    def insert(self, index, item):
        self.items.insert(index, item)
        if self.selected is None:
            self.selected = 0
            self._notifyselectionchanged()
        elif self.selected>=index:
            self.selected += 1
            self._notifyselectionchanged()
        self._updatetop()

    def append(self, item):
        self.items.append(item)
        if self.selected is None:
            self.selected = 0
            self._notifyselectionchanged()
        self._updatetop()

    def remove(self, item):
        for i in range(len(self)):
            if self[i] is item:
                del self[i]
                return

    def sort(self, func=lambda x,y: cmp(x.name,y.name)):
        self.items.sort(func)
        self._notifyselectionchanged()

    # slist specific methods

    # helper routines

    def _notifyselectionchanged(self):
        "helper routine, which issues a selectionchanged event, if list is active"
        
        if self.active:
            events.ec.notify(events.selectionchanged(self.getselected()))
    
    def _updatetop(self):
        "helper routine, which updates self.top"
        
        if len(self)<=self.height:
            self.top = 0
            return
        
        if self.selected is not None and self.selected<self.top:
            self.top = self.selected
            return

        if self.selected is not None and self.selected>self.top+self.height-1:
            self.top = self.selected-self.height+1
            return

    # slist specific methods

    def clear(self):
        "clear list"
        self.items = []
        self.selected = None
        self.top = 0
        self._notifyselectionchanged()

    def insertitem(self, item):
        "insert item at alphabetically correct position"
        for i in range(len(self)):
            if self[i].name>=item.name:
                self.insert(i-1, item)
                break
        else:
            self.append(item)

    def getselected(self):
        "return currently selected item"
        if self.selected is not None:
            return self.items[self.selected]
        return None

    def deleteselected(self):
        "delete currently selected item"
        if self.selected is not None:
            del self[self.selected]

    def selectbysearchstring(self, searchstring):
        "select next entry matching searchstring"
        if len(self)>0:
            if self.selected is None:
                first = 0
            else:
                first = self.selected

                searchstring = searchstring.lower()
            
                for i in range(first+1, len(self)) + range(first):
                    if self[i].name.lower().find(searchstring)!=-1:
                        self.selected = i
                        self._notifyselectionchanged()
                        self._updatetop()
                        break

    def selectbyletter(self, letter):
        "select next entry beginning with letter"
        if len(self)>0:
            if self.selected is None:
                first = 0
            else:
                first = self.selected

                letter = letter.lower()
            
                for i in range(first+1, len(self)) + range(first):
                    if self[i].name.lower().startswith(letter):
                        self.selected = i
                        self._notifyselectionchanged()
                        self._updatetop()
                        break

    def selectrelative(self, dist):
        "change selection relatively by dist"
        if len(self)>0:
            self.selected += dist
            self.selected = max(self.selected,0)
            self.selected = min(self.selected, len(self)-1)
            self._notifyselectionchanged()
            self._updatetop()
                
    def selectfirst(self):
        "select first item of list"
        if len(self)>0:
            self.selected = 0
            self._notifyselectionchanged()
            self._updatetop()
            
    def selectlast(self):
        "select last item of list"
        if len(self)>0:
            self.selected = len(self)-1
            self._notifyselectionchanged()
            self._updatetop()

    def selectnext(self):
        "select next item of list"
        self.selectrelative(1)

    def selectprev(self):
        "select previous item of list"
        self.selectrelative(-1)

    def selectnextpage(self):
        "select next page of list"
        self.selectrelative(self.height-1)

    def selectprevpage(self):
        "select previous page of list"
        self.selectrelative(-(self.height-1))
            
    def moveitemup(self):
        "move selected item up, if not first"
        if self.selected is not None and self.selected>0:
            self.items[self.selected-1], self.items[self.selected] = \
            self.items[self.selected], self.items[self.selected-1]
            self.selected -= 1
            self._updatetop()

    def moveitemdown(self):
        "move selected item down, if not last"
        if self.selected is not None and self.selected<len(self)-1:
            self.items[self.selected], self.items[self.selected+1] = \
            self.items[self.selected+1], self.items[self.selected]
            self.selected += 1
            self._updatetop()


