#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import shelve

import item
import events

from helper import debug

class songdb:
    def __init__(self, songdbbase=None):
        self.songdbbase = songdbbase
        
        if self.songdbbase:
            self.songs = shelve.open(self.songdbbase + "_songs.db")
            self.artists = shelve.open(self.songdbbase + "_artists.db")
            
            debug("songdb: %d songs and %d artists registered\n" %
                  (len(self.songs), len(self.artists)))
        else:
            self.songs = {}
            self.artists = {}

        # invalidate cache of songlist
        self.songlist = None

        events.ec.subscribe(events.songupdate, self.songupdate)

    def close(self):
        if self.songdbbase:
            self.songs.close()
            self.artists.close()

    def queryregister(self, path):
        """get song info from database or insert new one"""
        
        if self.songs.has_key(path):
            song = self.songs[path]
        else:
            debug("songdb: new song: %s\n" % path)
            song = self.songs[path] = item.song(path)
            self.songlist = None

        # insert song in artist and album database, if neccesary
        if not self.artists.has_key(song.artist):
            newartist = item.artist(song.artist)
            self.artists[song.artist] = newartist
            events.ec.notify(events.artistaddedtosongdb(newartist))
            
        if not self.artists[song.artist].albums.has_key(song.album):
            artist = self.artists[song.artist]
            newalbum = item.album(song.artist, song.album)
            artist.albums[song.album] = newalbum
            self.artists[song.artist] = artist

            events.ec.notify(events.albumaddedtosongdb(newalbum))
            
        if path not in self.artists[song.artist].albums[song.album].songs:
            artist = self.artists[song.artist]
            artist.albums[song.album].songs.append(path)
            self.artists[song.artist] = artist
            
            # note that for performance reasons, we don't send notifications
            # of song additions
            
        return song

    def updatesong(self, song):
        """updates entry of given song"""
        self.songs[song.path] = song
        
    def getsong(self, path):
        """returns song entry with given path"""
        return self.songs.get(path)

    def getartists(self):
        """return all stored artists"""
        return map(self.artists.get, self.artists.keys())

    def getalbums(self, artist=None):
        """return albums of a given artist.

        artist has to be a string. If it is none, all stored
        albums are returned
        """
        if artist is None:
            albums = []
            for artist in self.getartists():
                for album in self.getalbums(artist.name):
                    albums.append(album)
            return albums
        else:
            return self.artists[artist].albums.values()

    def getsongs(self, artist=None, album=None):
        """ returns song of given artist and given album
        
        artist and album are either strings or None
        If they are not given all artists and/or albums are chosen
        """
        
        if artist is None and album is None:
            # return all songs in songdb
            # we use the cached result in self.songlist, if present
            if self.songlist:
                return self.songlist
            else:
                self.songlist = map(self.getsong, self.songs.keys())
                return self.songlist
        elif album is None:
            # return all songs of a given artist
            songs = []
            for album in self.getalbums(artist):
                for song in self.getsongs(artist, album.name):
                    songs.append(song)
            return songs
        elif artist is None:
            # return all songs which are on albums with the given name
            return [ aalbum for aalbum in self.getalbums()
                            if aalbum.name==album ]
        else:
            # return all songs on a given album of a given artist
            return map(self.getsong,
                       self.artists[artist].albums[album].songs)
            
    def registerdirtree(self, dir):
        for name in os.listdir(dir):
            try:
                path = os.path.join(dir, name)
                if os.path.isdir(path):
                    events.ec.notify(events.timer())
                    self.registerdirtree(path)
                elif name.endswith(".mp3"):
                    self.queryregister(path)
            except: pass

    # event handler

    def songupdate(self, event):
        self.updatesong(event.song)


class songautoregisterer:
    def __init__(self, db, basedir):
        self.db = db
        self.basedir = basedir
        self.state = [(basedir, os.listdir(basedir), 0)]
        
        events.ec.subscribe(events.timer, self.timer)

    def timer(self, event):
        state = self.state.pop()
        countdown = 5
#        debug("songautoregisterer: %sprocessing %d items in %s starting from %d (depth %d)\n" %
#              ( " " * len(self.state),
#                len(state[1]),
#                state[0],
#                state[2]+1,
#                len(self.state)+1))
        for nr in range(state[2], len(state[1])):
            name = state[1][nr]
            path = os.path.join(state[0], name)
            
            if os.path.isdir(path):
                self.state.append((state[0], state[1], nr+1))
                try:
                    self.state.append((path, os.listdir(path), 0))
                except IOError:
                    pass
                return
            elif name.endswith(".mp3"):
                self.db.queryregister(path)
                countdown -= 1
                if countdown==0:
                    self.state.append((state[0], state[1], nr+1))
                    return

        if len(self.state)==0:
            debug("songautoregisterer: exit\n")
            events.ec.unsubscribe(events.timer, self.timer)
        
